@@ -2,7 +2,7 @@
 
 __author__ = "http://www.gemalto.com"
 
-Copyright 2001-2008 gemalto
+Copyright 2001-2009 gemalto
 Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
 
 This file is part of pyscard.
@@ -70,10 +70,12 @@ class CardConnection(Observable):
         """Remove a CardConnection observer."""
         Observable.deleteObserver( self, observer )
 
-    def connect( self, protocol=None ):
+    def connect( self, protocol=None, mode=None ):
         """Connect to card.
         protocol: a bit mask of the protocols to use, from CardConnection.T0_protocol, CardConnection.T1_protocol,
         CardConnection.RAW_protocol, CardConnection.T15_protocol
+
+        mode: passed as-is to the PC/SC layer
         """
         Observable.setChanged( self )
         Observable.notifyObservers( self, CardConnectionEvent('connect') )
@@ -139,14 +141,27 @@ class CardConnection(Observable):
         Subclasses must override this method for implementing apdu transmission."""
         pass
 
+    def control( self, controlCode, bytes = [] ):
+        """Send a control command and buffer.  Internally calls doControl()
+        class method and notify observers upon command/response events.
+        Subclasses must override the doControl() class method.
 
+        controlCode: command code
+ 
+        bytes:       list of bytes to transmit
+        """
+        Observable.setChanged( self )
+        Observable.notifyObservers( self, CardConnectionEvent('command', [controlCode, bytes] ) )
+        data = self.doControl( controlCode, bytes )
+        Observable.setChanged( self )
+        Observable.notifyObservers( self, CardConnectionEvent( 'response', data ) )
+        if None!=self.errorcheckingchain:
+            self.errorcheckingchain[0]( data )
+        return data
 
+    def doControl( self, controlCode, bytes ):
+        """Performs the command control.
 
-
-
-
-
-
-
-
+        Subclasses must override this method for implementing control."""
+        pass
 
