@@ -4,7 +4,7 @@ Sample for python PCSC wrapper module: get card ATR in first pcsc reader
 
 __author__ = "http://www.gemalto.com"
 
-Copyright 2001-2008 gemalto
+Copyright 2001-2009 gemalto
 Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
 
 This file is part of pyscard.
@@ -25,19 +25,20 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 """
 
 from smartcard.scard import *
+import smartcard.util
 
 try:
     hresult, hcontext = SCardEstablishContext( SCARD_SCOPE_USER )
     if hresult!=0:
-        raise  'Failed to establish context: ' + SCardGetErrorMessage(hresult)
+        raise error, 'Failed to establish context: ' + SCardGetErrorMessage(hresult)
     print 'Context established!'
 
     try:
         hresult, readers = SCardListReaders( hcontext, [] )
         if hresult!=0:
-            raise  'Failed to list readers: ' + SCardGetErrorMessage(hresult)
+            raise error, 'Failed to list readers: ' + SCardGetErrorMessage(hresult)
         if len(readers)<1:
-            raise  'No smart card readers'
+            raise Exception('No smart card readers')
         print 'PCSC Readers:', readers
 
         for reader in readers:
@@ -58,10 +59,7 @@ try:
                     print 'Reader:', reader
                     print 'State:', state
                     print 'Protocol:', protocol
-                    print 'ATR:',
-                    for i in xrange(len(atr)):
-                        print "0x%.2X" % atr[i],
-                    print ""
+                    print 'ATR:', smartcard.util.toHexString(atr, smartcard.util.HEX)
 
                 finally:
                     hresult = SCardDisconnect( hcard, SCARD_UNPOWER_CARD )
@@ -73,7 +71,7 @@ try:
     finally:
         hresult = SCardReleaseContext( hcontext )
         if hresult!=0:
-            raise  'Failed to release context: ' + SCardGetErrorMessage(hresult)
+            raise error, 'Failed to release context: ' + SCardGetErrorMessage(hresult)
         print 'Released context.'
 
 except:
