@@ -4,7 +4,7 @@ Sample for python PCSC wrapper module: List card attributes
 
 __author__ = "http://www.gemalto.com"
 
-Copyright 2001-2008 gemalto
+Copyright 2001-2009 gemalto
 Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
 
 This file is part of pyscard.
@@ -26,109 +26,67 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 import struct
 from smartcard.scard import *
+import smartcard.util
 
-if 'winscard'==resourceManager:
+if 'pcsclite-tiger' == resourceManagerSubType:
+    attributes = {}
+else:
     attributes = {
-        SCARD_ATTR_VENDOR_NAME              : 'SCARD_ATTR_VENDOR_NAME'              ,
-        SCARD_ATTR_VENDOR_IFD_TYPE          : 'SCARD_ATTR_VENDOR_IFD_TYPE'          ,
-        SCARD_ATTR_VENDOR_IFD_VERSION       : 'SCARD_ATTR_VENDOR_IFD_VERSION'       ,
-        SCARD_ATTR_VENDOR_IFD_SERIAL_NO     : 'SCARD_ATTR_VENDOR_IFD_SERIAL_NO'     ,
+        SCARD_ATTR_ATR_STRING               : 'SCARD_ATTR_ATR_STRING'               ,
         SCARD_ATTR_CHANNEL_ID               : 'SCARD_ATTR_CHANNEL_ID'               ,
+        SCARD_ATTR_CHARACTERISTICS          : 'SCARD_ATTR_CHARACTERISTICS'          ,
+        SCARD_ATTR_CURRENT_BWT              : 'SCARD_ATTR_CURRENT_BWT'              ,
+        SCARD_ATTR_CURRENT_CLK              : 'SCARD_ATTR_CURRENT_CLK'              ,
+        SCARD_ATTR_CURRENT_CWT              : 'SCARD_ATTR_CURRENT_CWT'              ,
+        SCARD_ATTR_CURRENT_D                : 'SCARD_ATTR_CURRENT_D'                ,
+        SCARD_ATTR_CURRENT_EBC_ENCODING     : 'SCARD_ATTR_CURRENT_EBC_ENCODING'     ,
+        SCARD_ATTR_CURRENT_F                : 'SCARD_ATTR_CURRENT_F'                ,
+        SCARD_ATTR_CURRENT_IFSC             : 'SCARD_ATTR_CURRENT_IFSC'             ,
+        SCARD_ATTR_CURRENT_IFSD             : 'SCARD_ATTR_CURRENT_IFSD'             ,
+        SCARD_ATTR_CURRENT_IO_STATE         : 'SCARD_ATTR_CURRENT_IO_STATE'         ,
+        SCARD_ATTR_CURRENT_N                : 'SCARD_ATTR_CURRENT_N'                ,
+        SCARD_ATTR_CURRENT_PROTOCOL_TYPE    : 'SCARD_ATTR_CURRENT_PROTOCOL_TYPE'    ,
+        SCARD_ATTR_CURRENT_W                : 'SCARD_ATTR_CURRENT_W'                ,
         SCARD_ATTR_DEFAULT_CLK              : 'SCARD_ATTR_DEFAULT_CLK'              ,
-        SCARD_ATTR_MAX_CLK                  : 'SCARD_ATTR_MAX_CLK'                  ,
         SCARD_ATTR_DEFAULT_DATA_RATE        : 'SCARD_ATTR_DEFAULT_DATA_RATE'        ,
+        SCARD_ATTR_DEVICE_FRIENDLY_NAME_A   : 'SCARD_ATTR_DEVICE_FRIENDLY_NAME_A'   ,
+        SCARD_ATTR_DEVICE_FRIENDLY_NAME_W   : 'SCARD_ATTR_DEVICE_FRIENDLY_NAME_W'   ,
+        SCARD_ATTR_DEVICE_IN_USE            : 'SCARD_ATTR_DEVICE_IN_USE'            ,
+        SCARD_ATTR_DEVICE_SYSTEM_NAME_A     : 'SCARD_ATTR_DEVICE_SYSTEM_NAME_A'     ,
+        SCARD_ATTR_DEVICE_SYSTEM_NAME_W     : 'SCARD_ATTR_DEVICE_SYSTEM_NAME_W'     ,
+        SCARD_ATTR_DEVICE_UNIT              : 'SCARD_ATTR_DEVICE_UNIT'              ,
+        SCARD_ATTR_ESC_AUTHREQUEST          : 'SCARD_ATTR_ESC_AUTHREQUEST'          ,
+        SCARD_ATTR_ESC_CANCEL               : 'SCARD_ATTR_ESC_CANCEL'               ,
+        SCARD_ATTR_ESC_RESET                : 'SCARD_ATTR_ESC_RESET'                ,
+        SCARD_ATTR_EXTENDED_BWT             : 'SCARD_ATTR_EXTENDED_BWT'             ,
+        SCARD_ATTR_ICC_INTERFACE_STATUS     : 'SCARD_ATTR_ICC_INTERFACE_STATUS'     ,
+        SCARD_ATTR_ICC_PRESENCE             : 'SCARD_ATTR_ICC_PRESENCE'             ,
+        SCARD_ATTR_ICC_TYPE_PER_ATR         : 'SCARD_ATTR_ICC_TYPE_PER_ATR'         ,
+        SCARD_ATTR_MAXINPUT                 : 'SCARD_ATTR_MAXINPUT'                 ,
+        SCARD_ATTR_MAX_CLK                  : 'SCARD_ATTR_MAX_CLK'                  ,
         SCARD_ATTR_MAX_DATA_RATE            : 'SCARD_ATTR_MAX_DATA_RATE'            ,
         SCARD_ATTR_MAX_IFSD                 : 'SCARD_ATTR_MAX_IFSD'                 ,
         SCARD_ATTR_POWER_MGMT_SUPPORT       : 'SCARD_ATTR_POWER_MGMT_SUPPORT'       ,
-        SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE : 'SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE' ,
+        SCARD_ATTR_SUPRESS_T1_IFS_REQUEST   : 'SCARD_ATTR_SUPRESS_T1_IFS_REQUEST'   ,
         SCARD_ATTR_USER_AUTH_INPUT_DEVICE   : 'SCARD_ATTR_USER_AUTH_INPUT_DEVICE'   ,
-        SCARD_ATTR_CHARACTERISTICS          : 'SCARD_ATTR_CHARACTERISTICS'          ,
-        SCARD_ATTR_CURRENT_PROTOCOL_TYPE    : 'SCARD_ATTR_CURRENT_PROTOCOL_TYPE'    ,
-        SCARD_ATTR_CURRENT_CLK              : 'SCARD_ATTR_CURRENT_CLK'              ,
-        SCARD_ATTR_CURRENT_F                : 'SCARD_ATTR_CURRENT_F'                ,
-        SCARD_ATTR_CURRENT_D                : 'SCARD_ATTR_CURRENT_D'                ,
-        SCARD_ATTR_CURRENT_N                : 'SCARD_ATTR_CURRENT_N'                ,
-        SCARD_ATTR_CURRENT_W                : 'SCARD_ATTR_CURRENT_W'                ,
-        SCARD_ATTR_CURRENT_IFSC             : 'SCARD_ATTR_CURRENT_IFSC'             ,
-        SCARD_ATTR_CURRENT_IFSD             : 'SCARD_ATTR_CURRENT_IFSD'             ,
-        SCARD_ATTR_CURRENT_BWT              : 'SCARD_ATTR_CURRENT_BWT'              ,
-        SCARD_ATTR_CURRENT_CWT              : 'SCARD_ATTR_CURRENT_CWT'              ,
-        SCARD_ATTR_CURRENT_EBC_ENCODING     : 'SCARD_ATTR_CURRENT_EBC_ENCODING'     ,
-        SCARD_ATTR_EXTENDED_BWT             : 'SCARD_ATTR_EXTENDED_BWT'             ,
-        SCARD_ATTR_ICC_PRESENCE             : 'SCARD_ATTR_ICC_PRESENCE'             ,
-        SCARD_ATTR_ICC_INTERFACE_STATUS     : 'SCARD_ATTR_ICC_INTERFACE_STATUS'     ,
-        SCARD_ATTR_CURRENT_IO_STATE         : 'SCARD_ATTR_CURRENT_IO_STATE'         ,
-        SCARD_ATTR_ATR_STRING               : 'SCARD_ATTR_ATR_STRING'               ,
-        SCARD_ATTR_ICC_TYPE_PER_ATR         : 'SCARD_ATTR_ICC_TYPE_PER_ATR'         ,
-        SCARD_ATTR_ESC_RESET                : 'SCARD_ATTR_ESC_RESET'                ,
-        SCARD_ATTR_ESC_CANCEL               : 'SCARD_ATTR_ESC_CANCEL'               ,
-        SCARD_ATTR_ESC_AUTHREQUEST          : 'SCARD_ATTR_ESC_AUTHREQUEST'          ,
-        SCARD_ATTR_MAXINPUT                 : 'SCARD_ATTR_MAXINPUT'                 ,
-        SCARD_ATTR_DEVICE_UNIT              : 'SCARD_ATTR_DEVICE_UNIT'              ,
-        SCARD_ATTR_DEVICE_IN_USE            : 'SCARD_ATTR_DEVICE_IN_USE'            ,
-        SCARD_ATTR_DEVICE_FRIENDLY_NAME_A   : 'SCARD_ATTR_DEVICE_FRIENDLY_NAME_A'   ,
-        SCARD_ATTR_DEVICE_SYSTEM_NAME_A     : 'SCARD_ATTR_DEVICE_SYSTEM_NAME_A'     ,
-        SCARD_ATTR_DEVICE_FRIENDLY_NAME_W   : 'SCARD_ATTR_DEVICE_FRIENDLY_NAME_W'   ,
-        SCARD_ATTR_DEVICE_SYSTEM_NAME_W     : 'SCARD_ATTR_DEVICE_SYSTEM_NAME_W'     ,
-        SCARD_ATTR_SUPRESS_T1_IFS_REQUEST   : 'SCARD_ATTR_SUPRESS_T1_IFS_REQUEST'
+        SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE : 'SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE' ,
+        SCARD_ATTR_VENDOR_IFD_SERIAL_NO     : 'SCARD_ATTR_VENDOR_IFD_SERIAL_NO'     ,
+        SCARD_ATTR_VENDOR_IFD_TYPE          : 'SCARD_ATTR_VENDOR_IFD_TYPE'          ,
+        SCARD_ATTR_VENDOR_IFD_VERSION       : 'SCARD_ATTR_VENDOR_IFD_VERSION'       ,
+        SCARD_ATTR_VENDOR_NAME              : 'SCARD_ATTR_VENDOR_NAME'
     }
-elif 'pcsclite'==resourceManager and 'pcsclite-tiger'!=resourceManagerSubType:
-    attributes = {
-        SCARD_ATTR_VENDOR_NAME              : 'SCARD_ATTR_VENDOR_NAME',
-        SCARD_ATTR_VENDOR_IFD_TYPE          : 'SCARD_ATTR_VENDOR_IFD_TYPE',
-        SCARD_ATTR_VENDOR_IFD_VERSION       : 'SCARD_ATTR_VENDOR_IFD_VERSION',
-        SCARD_ATTR_VENDOR_IFD_SERIAL_NO     : 'SCARD_ATTR_VENDOR_IFD_SERIAL_NO',
-        SCARD_ATTR_CHANNEL_ID               : 'SCARD_ATTR_CHANNEL_ID',
-        SCARD_ATTR_ASYNC_PROTOCOL_TYPES     : 'SCARD_ATTR_ASYNC_PROTOCOL_TYPES',
-        SCARD_ATTR_DEFAULT_CLK              : 'SCARD_ATTR_DEFAULT_CLK',
-        SCARD_ATTR_MAX_CLK                  : 'SCARD_ATTR_MAX_CLK',
-        SCARD_ATTR_DEFAULT_DATA_RATE        : 'SCARD_ATTR_DEFAULT_DATA_RATE',
-        SCARD_ATTR_MAX_DATA_RATE            : 'SCARD_ATTR_MAX_DATA_RATE',
-        SCARD_ATTR_MAX_IFSD                 : 'SCARD_ATTR_MAX_IFSD',
-        SCARD_ATTR_SYNC_PROTOCOL_TYPES      : 'SCARD_ATTR_SYNC_PROTOCOL_TYPES',
-        SCARD_ATTR_POWER_MGMT_SUPPORT       : 'SCARD_ATTR_POWER_MGMT_SUPPORT',
-        SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE : 'SCARD_ATTR_USER_TO_CARD_AUTH_DEVICE',
-        SCARD_ATTR_USER_AUTH_INPUT_DEVICE   : 'SCARD_ATTR_USER_AUTH_INPUT_DEVICE',
-        SCARD_ATTR_CHARACTERISTICS          : 'SCARD_ATTR_CHARACTERISTICS',
-        SCARD_ATTR_CURRENT_PROTOCOL_TYPE    : 'SCARD_ATTR_CURRENT_PROTOCOL_TYPE',
-        SCARD_ATTR_CURRENT_CLK              : 'SCARD_ATTR_CURRENT_CLK',
-        SCARD_ATTR_CURRENT_F                : 'SCARD_ATTR_CURRENT_F',
-        SCARD_ATTR_CURRENT_D                : 'SCARD_ATTR_CURRENT_D',
-        SCARD_ATTR_CURRENT_N                : 'SCARD_ATTR_CURRENT_N',
-        SCARD_ATTR_CURRENT_W                : 'SCARD_ATTR_CURRENT_W',
-        SCARD_ATTR_CURRENT_IFSC             : 'SCARD_ATTR_CURRENT_IFSC',
-        SCARD_ATTR_CURRENT_IFSD             : 'SCARD_ATTR_CURRENT_IFSD',
-        SCARD_ATTR_CURRENT_BWT              : 'SCARD_ATTR_CURRENT_BWT',
-        SCARD_ATTR_CURRENT_CWT              : 'SCARD_ATTR_CURRENT_CWT',
-        SCARD_ATTR_CURRENT_EBC_ENCODING     : 'SCARD_ATTR_CURRENT_EBC_ENCODING',
-        SCARD_ATTR_EXTENDED_BWT             : 'SCARD_ATTR_EXTENDED_BWT',
-        SCARD_ATTR_ICC_PRESENCE             : 'SCARD_ATTR_ICC_PRESENCE',
-        SCARD_ATTR_ICC_INTERFACE_STATUS     : 'SCARD_ATTR_ICC_INTERFACE_STATUS',
-        SCARD_ATTR_CURRENT_IO_STATE         : 'SCARD_ATTR_CURRENT_IO_STATE',
-        SCARD_ATTR_ATR_STRING               : 'SCARD_ATTR_ATR_STRING',
-        SCARD_ATTR_ICC_TYPE_PER_ATR         : 'SCARD_ATTR_ICC_TYPE_PER_ATR',
-        SCARD_ATTR_ESC_RESET                : 'SCARD_ATTR_ESC_RESET',
-        SCARD_ATTR_ESC_CANCEL               : 'SCARD_ATTR_ESC_CANCEL',
-        SCARD_ATTR_ESC_AUTHREQUEST          : 'SCARD_ATTR_ESC_AUTHREQUEST',
-        SCARD_ATTR_MAXINPUT                 : 'SCARD_ATTR_MAXINPUT',
-        SCARD_ATTR_DEVICE_UNIT              : 'SCARD_ATTR_DEVICE_UNIT',
-        SCARD_ATTR_DEVICE_IN_USE            : 'SCARD_ATTR_DEVICE_IN_USE',
-        SCARD_ATTR_DEVICE_FRIENDLY_NAME_A   : 'SCARD_ATTR_DEVICE_FRIENDLY_NAME_A',
-        SCARD_ATTR_DEVICE_SYSTEM_NAME_A     : 'SCARD_ATTR_DEVICE_SYSTEM_NAME_A',
-        SCARD_ATTR_DEVICE_FRIENDLY_NAME_W   : 'SCARD_ATTR_DEVICE_FRIENDLY_NAME_W',
-        SCARD_ATTR_DEVICE_SYSTEM_NAME_W     : 'SCARD_ATTR_DEVICE_SYSTEM_NAME_W',
-        SCARD_ATTR_SUPRESS_T1_IFS_REQUEST   : 'SCARD_ATTR_SUPRESS_T1_IFS_REQUEST'
-    }
-else:
-    attributes={}
+    if 'pcsclite'==resourceManager:
+        extra_attributes = {
+            SCARD_ATTR_ASYNC_PROTOCOL_TYPES     : 'SCARD_ATTR_ASYNC_PROTOCOL_TYPES',
+            SCARD_ATTR_SYNC_PROTOCOL_TYPES      : 'SCARD_ATTR_SYNC_PROTOCOL_TYPES',
+        }
+        attributes.update(extra_attributes)
 
 
 def printAttribute( attrib, value ):
     print '-----------------', attributes[attrib], '-----------------'
     print value
-    for j in xrange(len(value)):
-        print "0x%.2X" % value[j],
-    print ""
+    print smartcard.util.toHexString(value, smartcard.util.HEX)
     print apply( struct.pack, [ '<' + 'B' * len(value)] + value )
 
 
@@ -150,7 +108,7 @@ try:
 
         for reader in readers:
             hresult, hcard, dwActiveProtocol = SCardConnect(
-                hcontext, reader, SCARD_SHARE_SHARED, SCARD_PROTOCOL_T0 )
+                hcontext, reader, SCARD_SHARE_SHARED, SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1 )
             if hresult!=0:
                 print error, 'Unable to connect: ' + SCardGetErrorMessage(hresult)
             else:
