@@ -2,7 +2,7 @@
 
 __author__ = "http://www.gemalto.com"
 
-Copyright 2001-2008 gemalto
+Copyright 2001-2009 gemalto
 Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
 
 This file is part of pyscard.
@@ -82,20 +82,30 @@ class PCSCCardConnection( CardConnection ):
             raise CardConnectionException( 'Failed to release context: ' + SCardGetErrorMessage(hresult) )
         CardConnection.__del__( self )
 
-    def connect( self, protocol=None ):
-        """Connect to the card. If protocol is not specified, connect with the default connection protocol."""
+    def connect( self, protocol=None, mode=None ):
+        """Connect to the card.
+        
+        If protocol is not specified, connect with the default connection protocol.
+        
+        If mode is not specified, connect with SCARD_SHARE_SHARED."""
         CardConnection.connect( self, protocol )
         pcscprotocol = translateprotocolmask( protocol )
         if 0==pcscprotocol: pcscprotocol = self.getProtocol()
 
+        if mode==None: mode = SCARD_SHARE_SHARED
         hresult, self.hcard, dwActiveProtocol = SCardConnect(
-            self.hcontext, str(self.reader), SCARD_SHARE_SHARED, pcscprotocol )
+            self.hcontext, str(self.reader), mode, pcscprotocol )
         if hresult!=0:
             self.hcard=None
             if SCARD_W_REMOVED_CARD==hresult:
                 raise NoCardException( 'Unable to connect: ' + SCardGetErrorMessage(hresult) )
             else:
-                raise CardConnectionException( 'Unable to connect with protocol: ' + dictProtocol[pcscprotocol] + ' ' + SCardGetErrorMessage(hresult) )
+                raise CardConnectionException( 'Unable to connect with protocol: ' + dictProtocol[pcscprotocol] + '. ' + SCardGetErrorMessage(hresult) )
+        protocol = 0
+        for p in dictProtocol:
+            if p == dwActiveProtocol:
+                protocol = eval("CardConnection.%s_protocol" % dictProtocol[p])
+        PCSCCardConnection.setProtocol(self, protocol)
 
     def disconnect( self ):
         """Disconnect from the card."""
@@ -137,6 +147,8 @@ class PCSCCardConnection( CardConnection ):
                     sw2 is status word 2, e.g. 0x1A
                     response are the response bytes excluding status words
         """
+        if None==protocol:
+            protocol = self.getProtocol()
         CardConnection.doTransmit( self, bytes, protocol )
         pcscprotocolheader = translateprotocolheader( protocol )
         if 0==pcscprotocolheader:
@@ -153,6 +165,23 @@ class PCSCCardConnection( CardConnection ):
         data = map(lambda x: (x+256)%256, response[:-2])
         return data, sw1, sw2
 
+    def doControl( self, controlCode, bytes=[] ):
+        """Transmit a control command to the reader and return response.
+
+        controlCode: control command
+
+        bytes:       command data to transmit (list of bytes)
+
+        return:      response are the response bytes (if any)
+        """
+        CardConnection.doControl( self, controlCode, bytes )
+        hresult, response = SCardControl( self.hcard, controlCode, bytes )
+        if hresult!=0:
+            raise CardConnectionException( 'Failed to control ' + SCardGetErrorMessage(hresult) )
+
+        data = map(lambda x: (x+256)%256, response)
+        return data
+
 if __name__ == '__main__':
     """Small sample illustrating the use of CardConnection."""
     SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
@@ -162,12 +191,4 @@ if __name__ == '__main__':
     cc.connect()
     print "%r %x %x" % cc.transmit( SELECT + DF_TELECOM )
 
-
-
-
-
-
-
-
-
-
+    print cc.control( 0, [] )
