@@ -2,7 +2,7 @@
 
 __author__ = "gemalto http://www.gemalto.com"
 
-Copyright 2001-2008 gemalto
+Copyright 2001-2009 gemalto
 Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
 
 This file is part of pyscard.
@@ -31,9 +31,7 @@ from smartcard.Exceptions import *
 from smartcard.pcsc.PCSCExceptions import *
 from smartcard.scard import *
 
-hcontext = PCSCContext().getContext()
-
-def __PCSCreaders__( groups=[] ):
+def __PCSCreaders__( hcontext, groups=[] ):
     """Returns the list of PCSC smartcard readers in PCSC group.
 
     If group is not specified, returns the list of all PCSC smartcard readers.
@@ -102,7 +100,9 @@ class PCSCReader( Reader ):
 
 def readers( groups=[] ):
     creaders=[]
-    for reader in __PCSCreaders__( groups ):
+    hcontext = PCSCContext().getContext()
+
+    for reader in __PCSCreaders__( hcontext, groups ):
         creaders.append( ReaderFactory.createReader( 'smartcard.pcsc.PCSCReader.PCSCReader', reader ) )
     return creaders
 
@@ -111,9 +111,8 @@ if __name__ == '__main__':
     DF_TELECOM = [0x7F, 0x10]
 
     creaders=readers()
-    cc = creaders[1].createConnection()
+    cc = creaders[0].createConnection()
     cc.connect()
     data, sw1, sw2 = cc.transmit( SELECT + DF_TELECOM )
     print "%X %X" % (sw1, sw2)
 
-
