@@ -1,5 +1,5 @@
 /*==============================================================================
-Copyright 2001-2008 gemalto
+Copyright 2001-2009 gemalto
 Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
 
 This file is part of pyscard.
@@ -498,6 +498,10 @@ void SCardHelper_AppendReaderStateListToPyObject(
             PyObject* ot = PyTuple_New( 3 );
             oReader = PyString_FromString( source->ars[i].szReader );
             oEventState = PyInt_FromLong( (SCARDDWORDARG)source->ars[i].dwEventState );
+            // ATR visibly not initialised
+            if ( source->ars[i].cbAtr > SCARD_ATR_LENGTH)
+                source->ars[i].cbAtr = 0;
+
             oAtr = PyList_New( source->ars[i].cbAtr );
             for(j=0; j<source->ars[i].cbAtr; j++)
             {
@@ -746,7 +750,7 @@ build a SCARDCONTEXT from a python SCARDCONTEXT
     // do we have a python long?
     if (!PyLong_Check(source))
     {
-        PyErr_SetString( PyExc_TypeError, "Expected a python long." );
+        PyErr_SetString( PyExc_TypeError, "Expected a python long as SCARDCONTEXT." );
         return 0;
     }
 
@@ -816,7 +820,7 @@ build a SCARDHANDLE from a python SCARDHANDLE
     // do we have a python long?
     if (!PyLong_Check(source))
     {
-        PyErr_SetString( PyExc_TypeError, "Expected a python long." );
+        PyErr_SetString( PyExc_TypeError, "Expected a python long as SCARDHANDLE." );
         return 0;
     }
 
@@ -886,7 +890,7 @@ build a SCARDDWORDARG from a python SCARDDWORDARG
     if( !PyLong_Check(source) && !PyInt_Check(source) )
     {
         PyErr_SetString( PyExc_TypeError, "Expected a python integer or long." );
-        return 0;
+        return -1;
     }
 
     #ifdef PCSCLITE
@@ -1226,6 +1230,3 @@ Initialize and clean-up memory logging on process attach and detach
 }
 #endif // WIN32
 
-
-
-
