@@ -1,5 +1,5 @@
 /*==============================================================================
-Copyright 2001-2008 gemalto
+Copyright 2001-2009 gemalto
 Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
 
 This file is part of pyscard.
@@ -27,6 +27,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifdef PCSCLITE
     #include <dlfcn.h>
+    #include <string.h>
 #endif // PCSCLITE
 
 #ifndef NULL
@@ -293,6 +294,136 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
     }
 #endif // __TIGER__
 
+#ifdef PCSCLITE
+///////////////////////////////////////////////////////////////////////////////
+// some pcsclite versions (e.g. on Max OS X Tiger) have no pcsc_stringify
+// this function was taken from pcsclite
+//
+char* _defaultPCSCSTRINGIFYERROR( SCARDRETCODE pcscError )
+{
+    static char strError[75];
+
+    switch(pcscError )
+    {
+        case SCARD_S_SUCCESS:
+            strncpy( strError, "Command successful.", sizeof( strError ) );
+            break;
+        case SCARD_E_CANCELLED:
+            strncpy( strError, "Command cancelled.", sizeof( strError ) );
+            break;
+        case SCARD_E_CANT_DISPOSE:
+            strncpy( strError, "Cannot dispose handle.", sizeof( strError ) );
+            break;
+        case SCARD_E_INSUFFICIENT_BUFFER:
+            strncpy( strError, "Insufficient buffer.", sizeof( strError ) );
+            break;
+        case SCARD_E_INVALID_ATR:
+            strncpy( strError, "Invalid ATR.", sizeof( strError ) );
+            break;
+        case SCARD_E_INVALID_HANDLE:
+            strncpy( strError, "Invalid handle.", sizeof( strError ) );
+            break;
+        case SCARD_E_INVALID_PARAMETER:
+            strncpy( strError, "Invalid parameter given.", sizeof( strError ) );
+            break;
+        case SCARD_E_INVALID_TARGET:
+            strncpy( strError, "Invalid target given.", sizeof( strError ) );
+            break;
+        case SCARD_E_INVALID_VALUE:
+            strncpy( strError, "Invalid value given.", sizeof( strError ) );
+            break;
+        case SCARD_E_NO_MEMORY:
+            strncpy( strError, "Not enough memory.", sizeof( strError ) );
+            break;
+        case SCARD_F_COMM_ERROR:
+            strncpy( strError, "RPC transport error.", sizeof( strError ) );
+            break;
+        case SCARD_F_INTERNAL_ERROR:
+            strncpy( strError, "Internal error.", sizeof( strError ) );
+            break;
+        case SCARD_F_UNKNOWN_ERROR:
+            strncpy( strError, "Unknown error.", sizeof( strError ) );
+            break;
+        case SCARD_F_WAITED_TOO_LONG:
+            strncpy( strError, "Waited too long.", sizeof( strError ) );
+            break;
+        case SCARD_E_UNKNOWN_READER:
+            strncpy( strError, "Unknown reader specified.", sizeof( strError ) );
+            break;
+        case SCARD_E_TIMEOUT:
+            strncpy( strError, "Command timeout.", sizeof( strError ) );
+            break;
+        case SCARD_E_SHARING_VIOLATION:
+            strncpy( strError, "Sharing violation.", sizeof( strError ) );
+            break;
+        case SCARD_E_NO_SMARTCARD:
+            strncpy( strError, "No smart card inserted.", sizeof( strError ) );
+            break;
+        case SCARD_E_UNKNOWN_CARD:
+            strncpy( strError, "Unknown card.", sizeof( strError ) );
+            break;
+        case SCARD_E_PROTO_MISMATCH:
+            strncpy( strError, "Card protocol mismatch.", sizeof( strError ) );
+            break;
+        case SCARD_E_NOT_READY:
+            strncpy( strError, "Subsystem not ready.", sizeof( strError ) );
+            break;
+        case SCARD_E_SYSTEM_CANCELLED:
+            strncpy( strError, "System cancelled.", sizeof( strError ) );
+            break;
+        case SCARD_E_NOT_TRANSACTED:
+            strncpy( strError, "Transaction failed.", sizeof( strError ) );
+            break;
+        case SCARD_E_READER_UNAVAILABLE:
+            strncpy( strError, "Reader is unavailable.", sizeof( strError ) );
+            break;
+        case SCARD_W_UNSUPPORTED_CARD:
+            strncpy( strError, "Card is not supported.", sizeof( strError ) );
+            break;
+        case SCARD_W_UNRESPONSIVE_CARD:
+            strncpy( strError, "Card is unresponsive.", sizeof( strError ) );
+            break;
+        case SCARD_W_UNPOWERED_CARD:
+            strncpy( strError, "Card is unpowered.", sizeof( strError ) );
+            break;
+        case SCARD_W_RESET_CARD:
+            strncpy( strError, "Card was reset.", sizeof( strError ) );
+            break;
+        case SCARD_W_REMOVED_CARD:
+            strncpy( strError, "Card was removed.", sizeof( strError ) );
+            break;
+        case SCARD_E_UNSUPPORTED_FEATURE:
+            strncpy( strError, "Feature not supported.", sizeof( strError ) );
+            break;
+        case SCARD_E_PCI_TOO_SMALL:
+            strncpy( strError, "PCI struct too small.", sizeof( strError ) );
+            break;
+        case SCARD_E_READER_UNSUPPORTED:
+            strncpy( strError, "Reader is unsupported.", sizeof( strError ) );
+            break;
+        case SCARD_E_DUPLICATE_READER:
+            strncpy( strError, "Reader already exists.", sizeof( strError ) );
+            break;
+        case SCARD_E_CARD_UNSUPPORTED:
+            strncpy( strError, "Card is unsupported.", sizeof( strError ) );
+            break;
+        case SCARD_E_NO_SERVICE:
+            strncpy( strError, "Service not available.", sizeof( strError ) );
+            break;
+        case SCARD_E_SERVICE_STOPPED:
+            strncpy( strError, "Service was stopped.", sizeof( strError ) );
+            break;
+        default:
+            snprintf(strError, sizeof(strError)-1, "Unkown error: %ld, 0x%08lx", (long)pcscError, (long unsigned int)pcscError);
+    };
+
+    // zero terminates string
+    strError[sizeof(strError)-1] = '\0';
+
+    return strError;
+}
+#endif
+
 WINSCARDAPI SCARDRETCODE
 WINAPI _defaultSCARDBEGINTRANSACTION(
     IN      SCARDHANDLE hCard)
@@ -451,9 +582,14 @@ SCARDSTATUSA                    mySCardStatusA                      = _defaultSC
 SCARDTRANSMIT                   mySCardTransmit                     = _defaultSCARDTRANSMIT;
 
 
-unsigned long myg_prgSCardT0Pci=0L;
-unsigned long myg_prgSCardT1Pci=0L;
-unsigned long myg_prgSCardRawPci=0L;
+#ifdef PCSCLITE
+    PCSCSTRINGIFYERROR myPcscStringifyError = _defaultPCSCSTRINGIFYERROR;
+#endif // PCSCLITE
+
+
+void * myg_prgSCardT0Pci=NULL;
+void * myg_prgSCardT1Pci=NULL;
+void * myg_prgSCardRawPci=NULL;
 
 long winscard_init(void)
 {
@@ -509,9 +645,9 @@ long winscard_init(void)
                 GETPROCADDRESS( SCARDTRANSMIT                   , SCardTransmit );
 
 
-                myg_prgSCardT0Pci   = (unsigned long)GetProcAddress( hinstDLL, "g_rgSCardT0Pci"  );
-                myg_prgSCardT1Pci   = (unsigned long)GetProcAddress( hinstDLL, "g_rgSCardT1Pci"  );
-                myg_prgSCardRawPci  = (unsigned long)GetProcAddress( hinstDLL, "g_rgSCardRawPci" );
+                myg_prgSCardT0Pci   = GetProcAddress( hinstDLL, "g_rgSCardT0Pci"  );
+                myg_prgSCardT1Pci   = GetProcAddress( hinstDLL, "g_rgSCardT1Pci"  );
+                myg_prgSCardRawPci  = GetProcAddress( hinstDLL, "g_rgSCardRawPci" );
 
             }
          }
@@ -536,7 +672,7 @@ long winscard_init(void)
         #ifdef __APPLE__
             lib = "/System/Library/Frameworks/PCSC.framework/PCSC";
         #else
-            lib = "libpcsclite.so";
+            lib = "libpcsclite.so.1";
         #endif
 
         if( bFirstCall )
@@ -561,6 +697,7 @@ long winscard_init(void)
                 GETPROCADDRESS( SCARDRELEASECONTEXT    , SCardReleaseContext    , SCardReleaseContext    );
                 GETPROCADDRESS( SCARDSTATUSA           , SCardStatusA           , SCardStatus            );
                 GETPROCADDRESS( SCARDTRANSMIT          , SCardTransmit          , SCardTransmit          );
+                SILENTGETPROCADDRESS( PCSCSTRINGIFYERROR     , PcscStringifyError   , pcsc_stringify_error   );
 
                 #ifndef __APPLE__
                     GETPROCADDRESS( SCARDCONTROL, SCardControl, SCardControl );
@@ -578,11 +715,10 @@ long winscard_init(void)
                     GETPROCADDRESS( SCARDSETATTRIB         , SCardSetAttrib         , SCardSetAttrib         );
                 #endif
 
-                myg_prgSCardT0Pci   = (unsigned long)dlsym( handle, "g_rgSCardT0Pci"  );
-                myg_prgSCardT1Pci   = (unsigned long)dlsym( handle, "g_rgSCardT1Pci"  );
-                myg_prgSCardRawPci  = (unsigned long)dlsym( handle, "g_rgSCardRawPci" );
+                myg_prgSCardT0Pci  = dlsym( handle, "g_rgSCardT0Pci"  );
+                myg_prgSCardT1Pci  = dlsym( handle, "g_rgSCardT1Pci"  );
+                myg_prgSCardRawPci = dlsym( handle, "g_rgSCardRawPci" );
 
-                dlclose( handle );
                 dlsym_error = dlerror();
                 if( NULL!= dlsym_error )
                 {
